==> adeptarch-0.0.2.tar.gz: lib/src/resource/resource_loader.dart <==
import 'package:adeptarch/src/resource/resource.dart';
import 'package:adeptarch/src/utils/adept_publish_subject.dart';
import 'package:chopper/chopper.dart';
import 'package:adeptasync/src/async.dart';
/**
 * A generic class that can provide a resource backed by both the sqlite database and the network.
 * <p>
 *
 * @param <DataType>
 */

typedef OnLoadFromCache<DataType> = Future<DataType> Function();
typedef OnLoadedFromCache<DataType> = Function(Resource<DataType> resource);
typedef LoadFromStorage = Future<bool> Function();
typedef OnLoadFromStorage<DataType> = Future<DataType> Function();
typedef OnLoadedFromStorage<DataType> = Function(Resource<DataType> resource);
typedef OnLoadFromStorageFailed = Future<Resource> Function(dynamic exception);
typedef CreateAPICall<DataType> = Future<Response<DataType>> Function();
typedef OnLoadedFromAPI<DataType> = Future<Resource> Function({Resource<DataType> resource, Response<DataType> response});
typedef OnLoadFromAPIFailed = Future<Resource> Function({dynamic exception, int code});
typedef OnFailed = Future<Resource> Function(dynamic exception);
typedef OnErrorManagement = Future<Resource> Function(Resource resource);

class ResourceLoader<DataType>
{
  AsyncTask _loaderTask;
  AdeptPublishSubject<Resource<DataType>> _publishSubject;
  DataType _dataLoaded;
  OnLoadFromCache<DataType> _onLoadFromCache;
  OnLoadedFromCache<DataType> _onLoadedFromCache;
  LoadFromStorage _loadFromStorage;
  OnLoadFromStorage<DataType> _onLoadFromStorage;
  OnLoadedFromStorage<DataType> _onLoadedFromStorage;
  OnLoadFromStorageFailed _onLoadFromStorageFailed;
  CreateAPICall<DataType> _createAPICall;
  OnLoadedFromAPI<DataType> _onLoadedFromAPI;
  OnLoadFromAPIFailed _onLoadFromAPIFailed;
  OnFailed _onFailed;
  OnErrorManagement _onErrorManagement;
  bool _publishResults = true;

  Future<DataType> onLoadFromCache(){
    return _onLoadFromCache != null ? _onLoadFromCache() : null;
  }

  void onLoadedFromCache(Resource<DataType> resource) {
    if(_onLoadedFromCache != null) return _onLoadedFromCache(resource);
    if(_publishResults && _publishSubject != null) _publishSubject.post(resource);
  }

  Future<bool> loadFromStorage() async => _loadFromStorage != null ? _loadFromStorage() : false;

  Future<DataType> onLoadFromStorage() => _onLoadFromStorage != null ? _onLoadFromStorage() : null;

  void onLoadedFromStorage(Resource<DataType> resource){
    if(_onLoadedFromStorage != null) return _onLoadedFromStorage(resource);
    if(_publishResults && _publishSubject != null) _publishSubject.post(resource);
  }

  Future<Resource> onLoadFromStorageFailed(dynamic exception) async
  {
    if(_onLoadFromStorageFailed != null) return _onLoadFromStorageFailed(exception);
    Resource<DataType> resource = Resource.error(publishSubjectHasValue ? _publishSubject.value.data : null,
        exception.toString(), exception: exception);
    if(_onErrorManagement != null) resource = await _onErrorManagement(resource);
    if(_publishResults && _publishSubject != null)
      _publishSubject.post(resource);
    return resource;
  }

  Future<Response<DataType>> createAPICall() => _createAPICall != null ? _createAPICall() : null;

  Future<Resource> onLoadedFromAPI({Resource<DataType> resource, Response<DataType> response}) async {
    if(_onLoadedFromAPI != null) return _onLoadedFromAPI(resource: resource, response: response);
    if(response != null && response.isSuccessful)
      return onLoadedFromAPI(resource: Resource.success(_dataLoaded = response.body));
    if(_publishResults && _publishSubject != null) _publishSubject.post(resource);
    return resource;
  }

  Future<Resource> onLoadFromAPIFailed({dynamic exception, int code}) async
  {
    if(_onLoadFromAPIFailed != null) return _onLoadFromAPIFailed(exception: exception, code: code);
    Resource<DataType> resource = Resource.error(publishSubjectHasValue ? _publishSubject.value.data : null,
        exception != null ? exception.toString() : null, code: code, exception: exception);
    if(_onErrorManagement != null) resource = await _onErrorManagement(resource);
    if(_publishResults && _publishSubject != null)
      _publishSubject.post(resource);
    return resource;
  }

  Future<Resource> onFailed(dynamic exception) async
  {
    if(_onFailed != null) return _onFailed(exception);
    Resource<DataType> resource = Resource.error(publishSubjectHasValue ? _publishSubject.value.data : null,
        exception != null ? exception.toString() : null, exception: exception);
    if(_onErrorManagement != null) resource = await _onErrorManagement(resource);
    if(_publishResults && _publishSubject != null)
      _publishSubject.post(resource);
    return resource;
  }

  ResourceLoader({
  AdeptPublishSubject<Resource<DataType>> publishSubject,
  OnLoadFromCache<DataType> onLoadFromCache,
  OnLoadedFromCache<DataType> onLoadedFromCache,
  LoadFromStorage loadFromStorage,
  OnLoadFromStorage<DataType> onLoadFromStorage,
  OnLoadedFromStorage<DataType> onLoadedFromStorage,
  OnLoadFromStorageFailed onLoadFromStorageFailed,
  CreateAPICall<DataType> createAPICall,
  OnLoadedFromAPI<DataType> onLoadedFromAPI,
  OnLoadFromAPIFailed onLoadFromAPIFailed,
  OnFailed onFailed,
  OnErrorManagement onErrorManagement,
  })
  {
    _publishSubject = publishSubject;
    _onLoadFromCache = onLoadFromCache;
    _onLoadedFromCache = onLoadedFromCache;
    _loadFromStorage = loadFromStorage;
    _onLoadFromStorage = onLoadFromStorage;
    _onLoadedFromStorage = onLoadedFromStorage;
    _onLoadFromStorageFailed = onLoadFromStorageFailed;
    _createAPICall = createAPICall;
    _onLoadedFromAPI = onLoadedFromAPI;
    _onLoadFromAPIFailed = onLoadFromAPIFailed;
    _onFailed = onFailed;
    _onErrorManagement = onErrorManagement;
    _loaderTask = AsyncTask();
  }

  Future _loadResource() async{
    Resource<DataType> resource;
    try
    {
      _dataLoaded = await onLoadFromCache();

      var call = createAPICall();

      if(await loadFromStorage())
      {
        try
        {
          await onLoadedFromCache(resource = Resource.loading(_dataLoaded));
          _dataLoaded = await onLoadFromStorage();
          await onLoadedFromStorage(resource = call == null ? Resource.success(_dataLoaded) : Resource.loading(_dataLoaded));
        }
        catch(e) { resource = await onLoadFromStorageFailed(e); }
      }
      else
        await onLoadedFromCache(resource = (call == null ? Resource.success(_dataLoaded) : Resource.loading(_dataLoaded)));

      try
      {
        if (call != null) {
          final Response<DataType> response = await call;
          if (response.isSuccessful)
            resource = await onLoadedFromAPI(response: response);
          else
            resource = await onLoadFromAPIFailed(code: response.statusCode);
        }
      }catch(e) { resource = await onLoadFromAPIFailed(exception: e); }
    }catch(e) { resource = await onFailed(e); }
    return resource;
  }

  void load() {
    _loaderTask.execute(doInBackground: ({List values}) => _loadResource());
  }

  bool get publishSubjectHasValue => _publishSubject != null && _publishSubject.value != null;
  bool get publishSubjectValueHasData => publishSubjectHasValue && _publishSubject.value.data != null;

  AdeptPublishSubject<Resource<DataType>> asPublishSubject() {
    _publishResults = true;
    if(_publishSubject == null) _publishSubject = new AdeptPublishSubject<Resource<DataType>>();
    if(_publishSubject.value == null) _publishSubject.post(Resource.loading(null));
    else _publishSubject.post(Resource.loading(_publishSubject.value.data));
    load();
    return _publishSubject;
  }

  Future<Resource<DataType>> asFuture() async{
    _publishResults = false;
    return await _loadResource();
  }

  bool isCancelled() => _loaderTask != null && _loaderTask.isCanceled;
  bool isRunning() => _loaderTask != null && _loaderTask.isRunning;
  void cancel() => isRunning() ? _loaderTask.cancel() : null;
}

==> adeptarch-0.0.3.tar.gz: lib/src/resource/resource_loader.dart <==
import 'package:adeptarch/src/resource/resource.dart';
import 'package:adeptarch/src/utils/adept_publish_subject.dart';
import 'package:chopper/chopper.dart';
import 'package:adeptasync/adeptasync.dart';
import 'dart:async';
/**
 * A generic class that can provide a resource backed by both the sqlite database and the network.
 * <p>
 *
 * @param <DataType>
 */

typedef OnLoadFromCache<DataType> = Future<DataType> Function();
typedef OnLoadedFromCache<DataType> = Function(Resource<DataType> resource);
typedef LoadFromStorage = Future<bool> Function();
typedef OnLoadFromStorage<DataType> = Future<DataType> Function();
typedef OnLoadedFromStorage<DataType> = Function(Resource<DataType> resource);
typedef OnLoadFromStorageFailed = Future<Resource> Function(dynamic exception);
typedef CreateAPICall<DataType> = Future<Response<DataType>> Function();
typedef OnLoadedFromAPI<DataType> = Future<Resource> Function({Resource<DataType> resource, Response<DataType> response});
typedef OnLoadFromAPIFailed = Future<Resource> Function({dynamic exception, int code});
typedef OnFailed = Future<Resource> Function(dynamic exception);
typedef OnErrorManagement = Future<Resource> Function(Resource resource);

class ResourceLoader<DataType>
{
  AsyncTask _loaderTask;
  AdeptPublishSubject<Resource<DataType>> _publishSubject;
  DataType _dataLoaded;
  OnLoadFromCache<DataType> _onLoadFromCache;
  OnLoadedFromCache<DataType> _onLoadedFromCache;
  LoadFromStorage _loadFromStorage;
  OnLoadFromStorage<DataType> _onLoadFromStorage;
  OnLoadedFromStorage<DataType> _onLoadedFromStorage;
  OnLoadFromStorageFailed _onLoadFromStorageFailed;
  CreateAPICall<DataType> _createAPICall;
  OnLoadedFromAPI<DataType> _onLoadedFromAPI;
  OnLoadFromAPIFailed _onLoadFromAPIFailed;
  OnFailed _onFailed;
  OnErrorManagement _onErrorManagement;
  bool _publishResults = true;

  Future<DataType> onLoadFromCache(){
    return _onLoadFromCache != null ? _onLoadFromCache() : null;
  }

  Future onLoadedFromCache(Resource<DataType> resource) {
    if(_onLoadedFromCache != null) return _onLoadedFromCache(resource);
    if(_publishResults && _publishSubject != null) _publishSubject.post(resource);
    return null;
  }

  Future<bool> loadFromStorage() async => _loadFromStorage != null ? _loadFromStorage() : false;

  Future<DataType> onLoadFromStorage() => _onLoadFromStorage != null ? _onLoadFromStorage() : null;

  Future onLoadedFromStorage(Resource<DataType> resource){
    if(_onLoadedFromStorage != null) return _onLoadedFromStorage(resource);
    if(_publishResults && _publishSubject != null) _publishSubject.post(resource);
    return null;
  }

  Future<Resource> onLoadFromStorageFailed(dynamic exception) async
  {
    if(_onLoadFromStorageFailed != null) return _onLoadFromStorageFailed(exception);
    Resource<DataType> resource = Resource.error(publishSubjectHasValue ? _publishSubject.value.data : null,
        exception.toString(), exception: exception);
    if(_onErrorManagement != null) resource = await _onErrorManagement(resource);
    if(_publishResults && _publishSubject != null)
      _publishSubject.post(resource);
    return resource;
  }

  Future<Response<DataType>> createAPICall() => _createAPICall != null ? _createAPICall() : null;

  Future<Resource> onLoadedFromAPI({Resource<DataType> resource, Response<DataType> response}) async {
    if(_onLoadedFromAPI != null) return _onLoadedFromAPI(resource: resource, response: response);
    if(response != null && response.isSuccessful)
      return onLoadedFromAPI(resource: Resource.success(_dataLoaded = response.body));
    if(_publishResults && _publishSubject != null) _publishSubject.post(resource);
    return resource;
  }

  Future<Resource> onLoadFromAPIFailed({dynamic exception, int code}) async
  {
    if(_onLoadFromAPIFailed != null) return _onLoadFromAPIFailed(exception: exception, code: code);
    Resource<DataType> resource = Resource.error(publishSubjectHasValue ? _publishSubject.value.data : null,
        exception != null ? exception.toString() : null, code: code, exception: exception);
    if(_onErrorManagement != null) resource = await _onErrorManagement(resource);
    if(_publishResults && _publishSubject != null)
      _publishSubject.post(resource);
    return resource;
  }

  Future<Resource> onFailed(dynamic exception) async
  {
    if(_onFailed != null) return _onFailed(exception);
    Resource<DataType> resource = Resource.error(publishSubjectHasValue ? _publishSubject.value.data : null,
        exception != null ? exception.toString() : null, exception: exception);
    if(_onErrorManagement != null) resource = await _onErrorManagement(resource);
    if(_publishResults && _publishSubject != null)
      _publishSubject.post(resource);
    return resource;
  }

  ResourceLoader({
  AdeptPublishSubject<Resource<DataType>> publishSubject,
  OnLoadFromCache<DataType> onLoadFromCache,
  OnLoadedFromCache<DataType> onLoadedFromCache,
  LoadFromStorage loadFromStorage,
  OnLoadFromStorage<DataType> onLoadFromStorage,
  OnLoadedFromStorage<DataType> onLoadedFromStorage,
  OnLoadFromStorageFailed onLoadFromStorageFailed,
  CreateAPICall<DataType> createAPICall,
  OnLoadedFromAPI<DataType> onLoadedFromAPI,
  OnLoadFromAPIFailed onLoadFromAPIFailed,
  OnFailed onFailed,
  OnErrorManagement onErrorManagement,
  })
  {
    _publishSubject = publishSubject;
    _onLoadFromCache = onLoadFromCache;
    _onLoadedFromCache = onLoadedFromCache;
    _loadFromStorage = loadFromStorage;
    _onLoadFromStorage = onLoadFromStorage;
    _onLoadedFromStorage = onLoadedFromStorage;
    _onLoadFromStorageFailed = onLoadFromStorageFailed;
    _createAPICall = createAPICall;
    _onLoadedFromAPI = onLoadedFromAPI;
    _onLoadFromAPIFailed = onLoadFromAPIFailed;
    _onFailed = onFailed;
    _onErrorManagement = onErrorManagement;
    _loaderTask = AsyncTask();
  }

  Future _loadResource() async{
    Resource<DataType> resource;
    try
    {
      _dataLoaded = await onLoadFromCache();

      var call = createAPICall();

      if(await loadFromStorage())
      {
        try
        {
          await onLoadedFromCache(resource = Resource.loading(_dataLoaded));
          _dataLoaded = await onLoadFromStorage();
          await onLoadedFromStorage(resource = call == null ? Resource.success(_dataLoaded) : Resource.loading(_dataLoaded));
        }
        catch(e) { resource = await onLoadFromStorageFailed(e); }
      }
      else
        await onLoadedFromCache(resource = (call == null ? Resource.success(_dataLoaded) : Resource.loading(_dataLoaded)));

      try
      {
        if (call != null) {
          final Response<DataType> response = await call;
          if (response.isSuccessful)
            resource = await onLoadedFromAPI(response: response);
          else
            resource = await onLoadFromAPIFailed(code: response.statusCode);
        }
      }catch(e) { resource = await onLoadFromAPIFailed(exception: e); }
    }catch(e) { resource = await onFailed(e); }
    return resource;
  }

  void load() {
    _loaderTask.execute(doInBackground: ({List values}) => _loadResource());
  }

  bool get publishSubjectHasValue => _publishSubject != null && _publishSubject.value != null;
  bool get publishSubjectValueHasData => publishSubjectHasValue && _publishSubject.value.data != null;

  AdeptPublishSubject<Resource<DataType>> asPublishSubject() {
    _publishResults = true;
    if(_publishSubject == null) _publishSubject = new AdeptPublishSubject<Resource<DataType>>();
    if(_publishSubject.value == null) _publishSubject.post(Resource.loading(null));
    else _publishSubject.post(Resource.loading(_publishSubject.value.data));
    load();
    return _publishSubject;
  }

  Future<Resource<DataType>> asFuture() async{
    _publishResults = false;
    return await _loadResource();
  }

  bool isCancelled() => _loaderTask != null && _loaderTask.isCanceled;
  bool isRunning() => _loaderTask != null && _loaderTask.isRunning;
  void cancel() => isRunning() ? _loaderTask.cancel() : null;
}
